#include <stdio.h>

int mini (int A[], int N);
int maxi (int A[], int N);
void tripermut (int A[], int N);
void affichage (int A[], int N);
int rechSequence (int val, int A[], int N);
int rechDicot (int val, int A[], int N);
int rechDicotRec (int val, int A[], int deb, int fin);
void insereVal (int val, int A[], int n);
int fusion (int A[], int n, int B[], int m, int FUS[]);


/* donne la valeur minimal contenue dans le tableau A de N positions */
int mini (int A[], int N)
{
    int pos_min;   /* position du pos_minimum */
    int i;

    pos_min=0;
    for (i=0; i<N; i++) {
        if (A[i]<A[pos_min]) pos_min=i;
    }

    return (A[pos_min]);
}


/* donne la valeur max contenue dans le tableau A de N positions */
int maxi (int A[], int N)
{
    int pos_max;   /* position du pos_maximum */
    int i;

    pos_max=0;
    for (i=0; i<N; i++) {
        if (A[i]>A[pos_max]) pos_max=i;
    }
    return (A[pos_max]);
}


/* Tri du tableau par propagation de l'lment maximal. */
void tripermut (int A[], int N)
{
    int i, j, tmp;

    for (i=0;i<N;i++)  {
        for (j=i+1;j<N;j++) {
            if (A[i]>A[j])  {
                tmp = A[i];
                A[i] = A[j];
                A[j] = tmp;
            }
        }
    }
}

/* Affichage du tableau */
void affichage (int A[], int N) {
    int i;
   printf("Tableau  :\n");
   for (i=0; i<N; i++)
        printf("%d ", A[i]);
   printf("\n");
}


/* recherche sequentielle d'une valeur val dans le tableau A.
 * resultat position de la valeur donnee ou -1, si elle n'est pas presente */
int rechSequence (int val, int A[], int N) {
    int i, pos_val;

    pos_val = -1;  //au debut, on sait pas si on va le trouver
    for (i=0; i<N; i++) {
        if (A[i] == val) { //on a trouve
            pos_val = i;
        }
    }
    return (pos_val);
}


/* recherche dicothomique (binaire) d'une valeur val dans le tableau A.
 * resultat position de la valeur donnee ou -1, si elle n'est pas presente */
int rechDicot (int val, int A[], int N) {
    int pos_val;
    int deb; //debut de la zone de recherche (1er position)
    int fin; //fin de la zone de recherche (derniere position)
    int mil; //position centrale (milieu) de la zone de recherche

    pos_val = -1;  //au debut, on sait pas si on va le trouver
    deb = 0;       //on commence par tout le tableau
    fin = N-1;

    do {                     //zone de recherche: deb - fin
        mil = (deb + fin)/2; //on prend la position au milieu
        printf ("deb %d fin %d mil %d pos %d \n", deb, fin, mil, pos_val);

        if (val == A[mil]) { //on a trouve
            pos_val = mil;
        }
        else if (val < A[mil]) { //val recherche est inferieur
            fin = mil - 1;       //on va rechercher la partie avant du tableau
        }
        else {                   //val recherche est superieur
            deb = mil + 1;       //on va rechercher la partie apres le tableau
        }
    } while (deb <= fin && pos_val == -1);
    // on termine si on croise debut et fin, ou si on a trouve la valeur

    return (pos_val);
}


/* recherche dicothomique (binaire) d'une valeur val dans le tableau A.
 * (algorithme recursive)
 *   int deb : debut de la zone de recherche (1er position)
 *   int fin : fin de la zone de recherche (derniere position)
 *
 * resultat position de la valeur donnee ou -1, si elle n'est pas presente */
int rechDicotRec (int val, int A[], int deb, int fin) {
    int pos_val;
    int mil; //position centrale (milieu) de la zone de recherche

    pos_val = -1;  //au debut, on sait pas si on va le trouver
    // on termine si on croise debut et fin
    if (deb <= fin) {
        mil = (deb + fin)/2; //on prend la position au milieu
        printf ("deb %d fin %d mil %d pos %d \n", deb, fin, mil, pos_val);

        if (val == A[mil]) { //on a trouve
            pos_val = mil;
        }
        else if (val < A[mil]) {
            //val recherche est inferieur, on va rechercher la partie avant du tableau
            pos_val = rechDicotRec (val, A, deb, mil-1);
        }
        else {
            //val recherche est superieur on va rechercher la partie apres le tableau
            pos_val = rechDicotRec (val, A, mil+1, fin);
        }
    }

    return (pos_val);
}

/* insere une valeur val a un tableau A de dimension N+1
 * qui contient N valeurs entires tries par ordre croissant
 * de maniere a obtenir un tableau de N+1 valeurs tries.
 */
void insereVal (int val, int A[], int n) {
   int i;

   /* Dplacer les lments plus grands que */
   /* val d'une position vers l'arrire.    */
   for (i=n; (i>0) && (A[i-1]>val); i--) {
       A[i]=A[i-1];
   }
   /* VAL est copi  la position du dernier */
   /* lment dplac. */
   A[i]=val;

}


/* Fusion des lments de A et B dans FUS de faon  ce que FUS
 * soit aussi tri. La taille de FUS doit etre au moins n+m.
 * On retourne la taille final de FUS (normalment n+m).
 */
int fusion (int A[], int n, int B[], int m, int FUS[]) {
    int iA, iB, iFUS;

    iA=0; iB=0; iFUS=0;
    while ((iA<n) && (iB<m)) {
        if(A[iA]<B[iB])  {
             FUS[iFUS++]=A[iA++];
        }
        else {
             FUS[iFUS++]=B[iB++];
        }
    }

    /* Si iA ou iB sont arrivs  la fin de leur tableau, */
    /* alors copier le reste de l'autre tableau.          */
    while (iA<n){
         FUS[iFUS++]=A[iA++];
    }
    while (iB<m) {
         FUS[iFUS++]=B[iB++];
    }

    return iFUS;
}


